@@ -1,5 +1,5 @@
 ;; shepherd.scm -- The daemon shepherd.
-;; Copyright (C) 2013-2014, 2016, 2018-2020, 2022-2024 Ludovic Courtès <ludo@gnu.org>
+;; Copyright (C) 2013-2014, 2016, 2018-2020, 2022-2025 Ludovic Courtès <ludo@gnu.org>
 ;; Copyright (C) 2002, 2003 Wolfgang Jährling <wolfgang@pro-linux.de>
 ;; Copyright (C) 2018 Carlo Zancanaro <carlo@zancanaro.id.au>
 ;; Copyright (C) 2018 Danny Milosavljevic <dannym@scratchpost.org>
@@ -347,9 +347,13 @@ fork in the child process."
     ((_ () <> ((binding value) ...))
      (let ((real-primitive-fork primitive-fork))
        (set! primitive-fork
-             (lambda ()
+             (lambda* (#:key preserve-replacement-bindings?)
+               ;; Like 'primitive-fork', but restore the original core Guile
+               ;; bindings in the child process, unless
+               ;; PRESERVE-REPLACEMENT-BINDINGS? is true.
                (let ((result (real-primitive-fork)))
-                 (when (zero? result)
+                 (when (and (zero? result)
+                            (not preserve-replacement-bindings?))
                    (set! binding value)
                    ...
                    (set! primitive-fork real-primitive-fork))
