@@ -1,5 +1,5 @@
 ;; logger.scm -- Logging service output.
-;; Copyright (C) 2022-2024 Ludovic Courtès <ludo@gnu.org>
+;; Copyright (C) 2022-2025 Ludovic Courtès <ludo@gnu.org>
 ;;
 ;; This file is part of the GNU Shepherd.
 ;;
@@ -26,6 +26,7 @@
                                    process?
                                    process-id
                                    register-service-logger
+                                   service-canonical-name
                                    service-running-value
                                    service-status)
   #:use-module (shepherd support)
@@ -163,6 +164,9 @@ not exist."
   (define lines
     (make-channel))
 
+  (define output
+    (open-log-file file))
+
   (lambda ()
     (spawn-fiber (line-reader input lines))
 
@@ -170,7 +174,7 @@ not exist."
       ;; Associate this logger with SERVICE.
       (register-service-logger service channel))
 
-    (let log ((output (open-log-file file))
+    (let log ((output output)
               (messages (ring-buffer history-size))
               (service service))
       (call-with-port output
@@ -251,10 +255,19 @@ with timestamps to @var{file}; return the logger's control channel.  Associate
 the logger with @var{service}.  The logger will maintain a ring buffer of up
 to @var{history-size} lines in memory."
   (let ((channel (make-channel)))
-    (spawn-fiber (service-file-logger channel file input
-                                      #:service service
-                                      #:history-size history-size))
-    channel))
+    (catch 'system-error
+      (lambda ()
+        (spawn-fiber (service-file-logger channel file input
+                                          #:service service
+                                          #:history-size history-size))
+        channel)
+      (lambda args
+        (local-output (l10n "Failed to open log file '~a' for ~a: ~a.")
+                      file
+                      (or (and=> service service-canonical-name)
+                          (l10n "<unknown service>"))
+                      (strerror (system-error-errno args)))
+        (apply throw args)))))
 
 (define* (service-builtin-logger channel command input
                                  #:key
