@@ -1,5 +1,5 @@
 ;; monitor.scm -- Monitoring service.
-;; Copyright (C) 2023-2024 Ludovic Courtès <ludo@gnu.org>
+;; Copyright (C) 2023-2025 Ludovic Courtès <ludo@gnu.org>
 ;;
 ;; This file is part of the GNU Shepherd.
 ;;
@@ -54,7 +54,9 @@
        (log-monitoring-stats)
        (loop period))
       (('set-period period)
-       (local-output (l10n "Monitoring logging period changed to ~a seconds.")
+       (local-output (l10n "Monitoring logging period changed to ~a second."
+                           "Monitoring logging period changed to ~a seconds."
+                           period)
                      period)
        (loop period)))))
 
