@@ -1,5 +1,5 @@
 # GNU Shepherd --- Test basic communication capabilities.
-# Copyright © 2013-2014, 2016-2019, 2022-2023 Ludovic Courtès <ludo@gnu.org>
+# Copyright © 2013-2014, 2016-2019, 2022-2024 Ludovic Courtès <ludo@gnu.org>
 # Copyright © 2016 Mathieu Lirzin <mthl@gnu.org>
 # Copyright © 2014 Alex Sassmannshausen <alex.sassmannshausen@gmail.com>
 #
@@ -84,15 +84,28 @@ shepherd_pid="`cat $pid`"
 
 kill -0 $shepherd_pid
 test -S "$socket"
-pristine_status=`$herd status root` # Prep for 'reload' test.
+pristine_status=`$herd status -n 0` # Prep for 'reload' test.
 echo $pristine_status | grep -E '(Start.*root|Stop.*test)'
 
+# This command should display shepherd's own PID.
+$herd status root | grep "PID: $(cat "$pid")"
+
+# The "Recent messages" bit should contain that phrase.
+$herd status root | grep "Service root started"
+
+# The "Custom actions" bit should list these and more.
+$herd status root | grep "actions: .*eval"
+$herd status root | grep "actions: .*load"
+$herd status root | grep "actions: .*halt"
+
 $herd graph | grep '"test-2" -> "test"'
 
 $herd start test
 test -f "$stamp"
 $herd status test | grep running
 
+$herd status root | grep "Service test started" # from "Recent messages"
+
 $herd stop test
 test -f "$stamp" && false
 
@@ -110,6 +123,7 @@ test -z "`$herd stop test 2>&1`"
 $herd disable test-2
 if $herd start test-2
 then false; else true; fi
+$herd start test-2 | grep "test-2 is currently disabled"
 
 $herd enable test-2
 $herd start test-2
@@ -158,6 +172,7 @@ $herd status broken | grep "stopped"
 # Check 'make-system-constructor' and 'make-system-destructor'.
 $herd start spawn-with-system
 $herd status spawn-with-system | grep running
+$herd status spawn-with-system | grep "starting from " # recent messages
 $herd stop spawn-with-system
 $herd status spawn-with-system | grep "stopped"
 
@@ -174,6 +189,9 @@ then false; else true; fi
 $herd doc root action an-action-that-does-not-exist 2>&1 | \
     grep "does not have an action 'an-action-that-does-not-exist'"
 
+$herd doc root list-actions | grep ^daemonize:
+$herd doc root list-actions | grep ^halt:
+
 # Loading nonexistent file.
 if $herd load root /does/not/exist.scm;
 then false; else true; fi
@@ -181,19 +199,18 @@ then false; else true; fi
 # Unload two services, make sure the other it still around.
 $herd unload root broken
 $herd unload root test
-$herd status | grep -e "- test-2"
+$herd status -n 0 | grep -e "- test-2"
 
 $herd reload root "$conf"
-test "`$herd status`" == "$pristine_status"
+test "`$herd status -n 0`" == "$pristine_status"
 
 # Dynamically loading code.
 
 mkdir -p "$confdir"
 cat > "$confdir/some-conf.scm" <<EOF
 (register-services
- (make <service>                ;deprecated
-   #:provides '(test-loaded)
-   #:start (const 'abc)
+ (service '(test-loaded)  ;passing a rest list, which is deprecated
+   #:start (λ _ (display "Greek letter λ\n") 'abc)
    #:stop (const #f)))
 EOF
 
@@ -203,23 +220,54 @@ then false; else true; fi
 $herd load root "$confdir/some-conf.scm"
 rm "$confdir/some-conf.scm"
 
+# First, a deprecation warning should have been logged.
+grep "register-services.*rest list.*deprecated" "$log"
+
 # The new service should be loaded now.
 $herd status test-loaded
 $herd status test-loaded | grep stopped
 
 $herd start test-loaded
 $herd status test-loaded | grep -i 'running.*abc'
+grep "Greek letter" "$log"
 $herd stop test-loaded
 $herd unload root test-loaded
 
+# Load a service meant to execute from a nonexistent directory.
+cat > "$confdir/some-conf.scm" <<EOF
+(register-services
+ (list (service '(test-run-from-nonexistent-directory)
+	 #:start (make-forkexec-constructor '("$(type -P pwd)")
+                                            #:directory "/does-not-exist")
+	 #:stop (make-kill-destructor)
+         #:respawn? #f)))
+EOF
+
+$herd load root "$confdir/some-conf.scm"
+
+# 'start' succeeds (because fork(2) does) but it should stop immediately after
+# with the process marked as having failed.
+$herd start test-run-from-nonexistent-directory
+until $herd status test-run-from-nonexistent-directory | \
+	grep 'exited with code 127'
+do sleep 0.5; done
+
+# Check that we're still talking to the same shepherd process: it used to be
+# that 'fork+exec-command' would leave the faulty child process behind it when
+# 'chdir' throws.
+for i in $(seq 1 10)
+do
+    test $($herd eval root '(getpid)' | grep '^[0-9]') -eq $shepherd_pid
+done
+$herd unload root test-run-from-nonexistent-directory
+
 # Load a service whose running value does not have a valid read syntax, and
 # make sure that the running value is clamped before being sent over the wire.
 cat > "$confdir/some-conf.scm" <<EOF
 (register-services
- (make <service>               ;deprecated
-   #:provides '(test-loaded)
-   #:start (const (if #f #f))  ;#<undefined>
-   #:stop (const #f)))
+ (list (service '(test-loaded)
+	 #:start (const (if #f #f))  ;#<undefined>
+	 #:stop (const #f))))
 EOF
 
 $herd load root "$confdir/some-conf.scm"
