@@ -1,5 +1,5 @@
 # GNU Shepherd --- Ensure file descriptors are not leaked to children.
-# Copyright © 2022, 2023 Ludovic Courtès <ludo@gnu.org>
+# Copyright © 2022-2024 Ludovic Courtès <ludo@gnu.org>
 #
 # This file is part of the GNU Shepherd.
 #
@@ -154,8 +154,8 @@ test $(cat "$fd_count") -eq 4
 $herd start forkexec-ctor
 $herd status forkexec-ctor
 
-pid="$($herd status forkexec-ctor | grep "Running value" \
-  | sed -e's/^.* \([0-9]\+\)\.$/\1/g')"
+pid="$($herd status forkexec-ctor | grep "PID: [0-9]\+" \
+  | sed -e's/^.* \([0-9]\+\)$/\1/g')"
 kill -0 "$pid"
 
 ls -l "/proc/$pid/fd"
