@@ -1,5 +1,5 @@
 # GNU Shepherd --- Test transient services.
-# Copyright © 2022, 2023 Ludovic Courtès <ludo@gnu.org>
+# Copyright © 2022-2024 Ludovic Courtès <ludo@gnu.org>
 #
 # This file is part of the GNU Shepherd.
 #
@@ -47,7 +47,8 @@ cat > "$conf" <<EOF
 				     (endpoint (make-socket-address
 						AF_INET
 						INADDR_LOOPBACK
-						$PORT))))
+						$PORT)))
+                                    #:service-name-stem "test-inetd")
     #:stop  (make-inetd-destructor))
   (service
     '(test-inetd6)
@@ -119,6 +120,23 @@ converse_with_echo_server ()
       (match (read-line sock) (\"done\" #t))"
 }
 
+$herd status test-inetd | grep "Inetd-style service"
+$herd status test-inetd | grep "127\.0\.0\.1:$PORT"
+
+# Initiate a conversation and make sure it creates a transient service with
+# correct process information.
+guile -c "
+  (define address (make-socket-address AF_INET INADDR_LOOPBACK $PORT))
+  (define sock (socket (sockaddr:fam address) SOCK_STREAM 0))
+  (connect sock address)
+  (sleep 300)" &
+client_pid=$!
+$herd status test-inetd-1
+$herd status test-inetd-1 | grep transient
+$herd status test-inetd-1 | grep "Command:.*$SHELL.*echo hello"
+$herd stop test-inetd-1
+kill "$client_pid" || true
+$herd status test-inetd-1 && false
 
 for i in $(seq 1 3)
 do
