@@ -1,5 +1,5 @@
 # GNU Shepherd --- Test respawnable services.
-# Copyright © 2013, 2014, 2016, 2023 Ludovic Courtès <ludo@gnu.org>
+# Copyright © 2013-2014, 2016, 2023-2024 Ludovic Courtès <ludo@gnu.org>
 #
 # This file is part of the GNU Shepherd.
 #
@@ -59,6 +59,9 @@ function assert_killed_service_is_respawned
 
     test "$old_pid" -ne "$new_pid"
     kill -0 "$new_pid"
+
+    $herd status "$2" | grep "Last respawned"
+    $herd status "$2" | grep "terminated with signal 15"
 }
 
 cat > "$conf"<<EOF
@@ -118,8 +121,8 @@ kill -0 `cat "$service2_pid"`
 # Now, kill the services, and make sure they both get respawned.
 for i in $(seq 1 10)
 do
-    assert_killed_service_is_respawned "$service1_pid"
-    assert_killed_service_is_respawned "$service2_pid"
+    assert_killed_service_is_respawned "$service1_pid" test1
+    assert_killed_service_is_respawned "$service2_pid" test2
 done
 
 # Make sure the respawnable service can be stopped.
@@ -140,4 +143,47 @@ $herd status test1 | grep disabled
 until $herd status test1 | grep stopped; do sleep 0.3; done
 
 cat $service2_pid
+
+# Respawning vs. replacing.
+#
+# Check that service respawn honors replacements: once a replacement has been
+# loaded, it is that replacement that should be spawned, not the original
+# service.
+
+cat > "$conf" <<EOF
+(register-services
+ (list (service
+	 '(test3 test3-v1)
+	 #:start (make-forkexec-constructor
+		  ;; Exit normally after a short delay.
+		  '("$SHELL" "-c" "echo test3 v1... ; sleep 0.2"))
+	 #:stop  (make-kill-destructor)
+	 #:respawn? #t)))
+
+(start-in-the-background '(test3))
+EOF
+
+$herd load root "$conf"
+until grep "Respawning test3" "$log"; do sleep 0.2; done
+$herd status test3 | grep "running"
+grep "test3 v1" "$log"
+
+# Load a replacement.  This should break the respawn loop by starting the
+# replacement, "test3 v2".
+cat > "$conf" <<EOF
+(register-services
+ (list (service
+	 '(test3 test3-v2)
+	 #:start (make-forkexec-constructor
+		  '("$SHELL" "-c" "echo test3 v2... ; sleep 180"))
+	 #:stop  (make-kill-destructor)
+	 #:respawn? #f)))
+EOF
+$herd load root "$conf"
+
+until grep "test3 v2" "$log"; do sleep 0.2; done
+$herd status test3-v1 && false
+$herd status test3-v2 | grep "running"
+$herd status test3 | grep "Will not be respawned"
+
 $herd stop root
